import logging

from spaceone.core import config
from spaceone.core.manager import BaseManager
from spaceone.secret.error import *

_LOGGER = logging.getLogger(__name__)


class SecretConnectorManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        connector = config.get_global("CONNECTORS")
        backend = config.get_global("BACKEND", "AWSSecretManagerConnector")
        try:
            _LOGGER.debug(f"[SecretConnectorManager] Create {backend}")
            self.secret_conn = self.locator.get_connector(backend)
        except Exception as e:
            _LOGGER.error(f"[SecretConnectorManager] not defined backend {backend}")
            raise ERROR_DEFINE_SECRET_BACKEND(backend=backend)

    def create_secret(self, secret_id, data):
        def _rollback(secret_id):
            _LOGGER.info(f"[ROLLBACK] Delete secret data in secret store : {secret_id}")
            self.secret_conn.delete_secret(secret_id)

        response = self.secret_conn.create_secret(secret_id, data)
        self.transaction.add_rollback(_rollback, secret_id)

    def update_secret(self, secret_id, data):
        self.secret_conn.update_secret(secret_id, data)

    def delete_secret(self, secret_id):
        self.secret_conn.delete_secret(secret_id)

    def get_secret(self, secret_id):
        return self.secret_conn.get_secret(secret_id)



from spaceone.core.error import *


class ERROR_DEFINE_SECRET_BACKEND(ERROR_BASE):
    _message = "Secret Backend is not defined. {backend}"


class ERROR_WRONG_ENCRYPT_ALGORITHM(ERROR_HANDLER_CONFIGURATION):
    _message = "EncryptAlgorithm({encrypt_algorithm}) is not supported."


class ERROR_NOT_EXIST_TRUST_SERVICE_ACCOUNT(ERROR_BASE):
    _message = "Trust type service account is not exist"


class ERROR_EXIST_RELATED_SECRET(ERROR_BASE):
    _message = "Related Secret is exist. (secret_id={secret_id})"


class ERROR_DIFF_SECRET_AND_TRUSTED_SECRET_ENCRYPTED(ERROR_BASE):
    _message = "The encryption algorithm of Secret and Trusted Secret are different."



import logging

from spaceone.core.manager import BaseManager
from spaceone.secret.model.user_secret_model import UserSecret

_LOGGER = logging.getLogger(__name__)


class UserSecretManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user_secret_model: UserSecret = self.locator.get_model("UserSecret")

    def create_user_secret(self, params):
        def _rollback(user_secret_vo):
            _LOGGER.info(
                f"[ROLLBACK] Delete user_secret : {user_secret_vo.name} ({user_secret_vo.user_secret_id})"
            )
            user_secret_vo.delete()

        user_secret_vo: UserSecret = self.user_secret_model.create(params)

        self.transaction.add_rollback(_rollback, user_secret_vo)

        return user_secret_vo

    def update_user_secret_by_vo(self, params, user_secret_vo):
        def _rollback(old_data):
            _LOGGER.info(
                f'[ROLLBACK] Revert Data : {old_data["name"]} ({old_data["user_secret_id"]})'
            )
            user_secret_vo.update(old_data)

        self.transaction.add_rollback(_rollback, user_secret_vo.to_dict())

        return user_secret_vo.update(params)

    @staticmethod
    def delete_user_secret_by_vo(user_secret_vo):
        user_secret_vo.delete()

    def get_user_secret(self, user_secret_id, domain_id, user_id=None):
        conditions = {
            "user_secret_id": user_secret_id,
            "domain_id": domain_id,
        }

        if user_id:
            conditions["user_id"] = user_id

        return self.user_secret_model.get(**conditions)

    def filter_user_secrets(self, **conditions):
        return self.user_secret_model.filter(**conditions)

    def list_user_secrets(self, query):
        return self.user_secret_model.query(**query)

    def stat_user_secrets(self, query):
        return self.user_secret_model.stat(**query)



import logging

from spaceone.core.manager import BaseManager
from spaceone.secret.model.secret_model import Secret

_LOGGER = logging.getLogger(__name__)


class SecretManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.secret_model: Secret = self.locator.get_model("Secret")

    def create_secret(self, params):
        def _rollback(secret_vo):
            _LOGGER.info(
                f"[ROLLBACK] Delete secret : {secret_vo.name} ({secret_vo.secret_id})"
            )
            secret_vo.delete()

        secret_vo: Secret = self.secret_model.create(params)

        self.transaction.add_rollback(_rollback, secret_vo)

        return secret_vo

    def update_secret_by_vo(self, params, secret_vo):
        def _rollback(old_data):
            _LOGGER.info(
                f'[ROLLBACK] Revert Data : {old_data["name"]} ({old_data["secret_id"]})'
            )
            secret_vo.update(old_data)

        self.transaction.add_rollback(_rollback, secret_vo.to_dict())

        return secret_vo.update(params)

    @staticmethod
    def delete_secret_by_vo(secret_vo):
        secret_vo.delete()

    def get_secret(self, secret_id, domain_id, workspace_id=None, user_projects=None):
        conditions = {
            "secret_id": secret_id,
            "domain_id": domain_id,
        }

        if workspace_id:
            conditions["workspace_id"] = workspace_id

        if user_projects:
            conditions["project_id"] = user_projects

        return self.secret_model.get(**conditions)

    def filter_secrets(self, **conditions):
        return self.secret_model.filter(**conditions)

    def list_secrets(self, query):
        return self.secret_model.query(**query)

    def stat_secrets(self, query):
        return self.secret_model.stat(**query)



# Database Settings
DATABASE_AUTO_CREATE_INDEX = True
DATABASES = {
    "default": {
        # 'db': '',
        # 'host': '',
        # 'port': 0,
        # 'username': '',
        # 'password': '',
        # 'ssl': False,
        # 'ssl_ca_certs': ''
    }
}

# Cache Settings
CACHES = {
    "default": {},
    "local": {
        "backend": "spaceone.core.cache.local_cache.LocalCache",
        "max_size": 128,
        "ttl": 300,
    },
}

# Handler Settings
HANDLERS = {
    # "authentication": [{
    #     "backend": "spaceone.core.handler.authentication_handler:SpaceONEAuthenticationHandler"
    # }],
    # "authorization": [{
    #     "backend": "spaceone.core.handler.authorization_handler:SpaceONEAuthorizationHandler"
    # }],
    # "mutation": [{
    #     "backend": "spaceone.core.handler.mutation_handler:SpaceONEMutationHandler"
    # }],
    # "event": []
}

# Connector Settings
BACKEND = "AWSSecretManagerConnector"
CONNECTORS = {
    "SpaceConnector": {
        "backend": "spaceone.core.connector.space_connector:SpaceConnector",
        "endpoints": {"identity": "grpc://identity:50051"},
    },
    "AWSSecretManagerConnector": {},
    "VaultConnector": {
        # 'url': 'http://vault:8200',
        # 'token': 'myroot'
    },
    "ConsulConnector": {"host": "consul", "port": 8500},
    "EtcdConnector": {"host": "localhost", "port": 2379},
    "MongoDBConnector": {
        "host": "localhost",
        "port": 27017,
        "username": "",
        "password": "",
    },
}

LOG = {
    "filters": {
        "masking": {
            "rules": {
                "Secret.create": ["data"],
                "Secret.update_data": ["data"],
                "TrustedSecret.create": ["data"],
                "TrustedSecret.update_data": ["data"],
            }
        }
    }
}



import logging
import functools
from spaceone.api.secret.v1 import user_secret_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.secret.model.user_secret_model import UserSecret

__all__ = ["UserSecretInfo", "UserSecretsInfo", "UserSecretDataInfo"]
_LOGGER = logging.getLogger(__name__)


def UserSecretDataInfo(secret_data):
    info = {
        "encrypted": secret_data.get("encrypted", False),
        "encrypt_options": change_struct_type(secret_data.get("encrypt_options", {})),
        "data": change_struct_type(secret_data["data"]),
    }

    return user_secret_pb2.UserSecretDataInfo(**info)


def UserSecretInfo(secret_vo: UserSecret, minimal=False):
    info = {
        "user_secret_id": secret_vo.user_secret_id,
        "name": secret_vo.name,
        "schema_id": secret_vo.schema_id,
        "provider": secret_vo.provider,
    }

    if minimal is False:
        info.update(
            {
                "tags": change_struct_type(secret_vo.tags),
                "user_id": secret_vo.user_id,
                "domain_id": secret_vo.domain_id,
                "created_at": utils.datetime_to_iso8601(secret_vo.created_at),
            }
        )

    return user_secret_pb2.UserSecretInfo(**info)


def UserSecretsInfo(secret_vos, total_count, **kwargs):
    results = list(map(functools.partial(UserSecretInfo, **kwargs), secret_vos))

    return user_secret_pb2.UserSecretsInfo(results=results, total_count=total_count)



import logging

from spaceone.core.service import *
from spaceone.core.service.utils import *

from spaceone.secret.manager.identity_manager import IdentityManager
from spaceone.secret.manager.user_secret_manager import UserSecretManager
from spaceone.secret.model.user_secret_model import UserSecret
from spaceone.secret.manager.secret_connector_manager import SecretConnectorManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class UserSecretService(BaseService):
    resource = "UserSecret"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user_secret_mgr: UserSecretManager = self.locator.get_manager(
            "UserSecretManager"
        )
        self.identity_mgr: IdentityManager = self.locator.get_manager("IdentityManager")

    @transaction(
        permission="secret:UserSecret.write",
        role_types=["USER"],
    )
    @check_required(["name", "data", "user_id", "domain_id"])
    def create(self, params):
        """Create user secret

        Args:
            params (dict): {
                'name': 'str',                  # required
                'data': 'dict',                 # required
                'schema_id': 'str',
                'tags': 'dict',
                'encrypted': 'bool',
                'encrypt_options': 'dict',
                'user_id': 'str',               # inherited from auth (required)
                'domain_id': 'str'              # inherited from auth (required)
            }

        Returns:
            user_secret_vo
        """

        user_secret_vo = self.user_secret_mgr.create_user_secret(params)

        secret_conn_mgr: SecretConnectorManager = self.locator.get_manager(
            "SecretConnectorManager"
        )
        secret_conn_mgr.create_secret(user_secret_vo.user_secret_id, params["data"])

        return user_secret_vo

    @transaction(
        permission="secret:UserSecret.write",
        role_types=["USER"],
    )
    @check_required(["user_secret_id", "user_id", "domain_id"])
    def update(self, params):
        """Update user secret

        Args:
            params (dict): {
                'user_secret_id': 'str',    # required
                'name': 'str' ,
                'tags': 'dict',
                'user_id': 'str',           # inherited from auth (required)
                'domain_id': 'str'          # inherited from auth (required)
            }

        Returns:
            user_secret_vo
        """

        user_id = params["user_id"]
        domain_id = params["domain_id"]
        user_secret_id = params["user_secret_id"]

        user_secret_vo: UserSecret = self.user_secret_mgr.get_user_secret(
            user_secret_id, domain_id, user_id
        )

        user_secret_vo = self.user_secret_mgr.update_user_secret_by_vo(
            params, user_secret_vo
        )

        return user_secret_vo

    @transaction(
        permission="secret:UserSecret.write",
        role_types=["USER"],
    )
    @check_required(["user_secret_id", "user_id", "domain_id"])
    def delete(self, params):
        """Delete user secret

        Args:
            params (dict): {
                'user_secret_id': 'str',        # required
                'user_id': 'str',               # inherited from auth (required)
                'domain_id': 'str'.             # inherited from auth (required)
            }

        Returns:
            None
        """

        user_secret_id = params["user_secret_id"]
        user_id = params["user_id"]
        domain_id = params["domain_id"]

        user_secret_vo = self.user_secret_mgr.get_user_secret(
            user_secret_id, domain_id, user_id
        )

        secret_conn_mgr: SecretConnectorManager = self.locator.get_manager(
            "SecretConnectorManager"
        )
        secret_conn_mgr.delete_secret(user_secret_id)

        self.user_secret_mgr.delete_user_secret_by_vo(user_secret_vo)

    @transaction(
        permission="secret:UserSecret.write",
        role_types=["USER"],
    )
    @check_required(["user_secret_id", "data", "user_id", "domain_id"])
    def update_data(self, params):
        """Update user secret data

        Args:
            params (dict): {
                'user_secret_id': 'str',            # required
                'schema_id': 'str',
                'data': 'dict',                     # required
                'encrypted': 'bool',
                'encrypt_options': 'dict',
                'user_id': 'str',                   # inherited from auth (required)
                'domain_id': 'str',                 # inherited from auth (required)
            }

        Returns:
            user_secret_data (dict)
        """

        user_id = params["user_id"]
        domain_id = params["domain_id"]
        user_secret_id = params["user_secret_id"]
        data = params["data"]

        user_secret_vo = self.user_secret_mgr.get_user_secret(
            user_secret_id, domain_id, user_id
        )
        self.user_secret_mgr.update_user_secret_by_vo(params, user_secret_vo)

        secret_conn_mgr: SecretConnectorManager = self.locator.get_manager(
            "SecretConnectorManager"
        )
        secret_conn_mgr.update_secret(user_secret_id, data)

    @transaction(exclude=["authentication", "authorization", "mutation"])
    @check_required(["user_secret_id", "domain_id"])
    def get_data(self, params):
        """Get user secret data

        Args:
            params (dict): {
                'user_secret_id': 'str',        # required
                'domain_id': 'str',             # required
            }

        Returns:
            user_secret_data (dict)
        """

        user_secret_id = params["user_secret_id"]
        domain_id = params["domain_id"]

        user_secret_vo: UserSecret = self.user_secret_mgr.get_user_secret(
            user_secret_id, domain_id
        )

        secret_conn_mgr: SecretConnectorManager = self.locator.get_manager(
            "SecretConnectorManager"
        )
        user_secret_data = secret_conn_mgr.get_secret(user_secret_id)

        return {
            "encrypted": user_secret_vo.encrypted,
            "encrypt_options": user_secret_vo.encrypt_options,
            "data": user_secret_data,
        }

    @transaction(
        permission="secret:UserSecret.read",
        role_types=["USER"],
    )
    @check_required(["user_secret_id", "user_id", "domain_id"])
    def get(self, params):
        """Get user secret

        Args:
            params (dict): {
                'user_secret_id': 'str',        # required
                'user_id': 'str',               # inherited from auth (required)
                'domain_id': 'str',             # inherited from auth (required)
            }

        Returns:
            user_secret_vo
        """

        return self.user_secret_mgr.get_user_secret(
            params["user_secret_id"],
            params["domain_id"],
            params["user_id"],
        )

    @transaction(
        permission="secret:UserSecret.read",
        role_types=["USER"],
    )
    @check_required(["user_id", "domain_id"])
    @append_query_filter(
        [
            "user_secret_id",
            "name",
            "schema_id",
            "provider",
            "user_id",
            "domain_id",
        ]
    )
    @append_keyword_filter(["user_secret_id", "name", "schema_id", "provider"])
    def list(self, params):
        """List user secrets

        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'user_secret_id': 'str',
                'name': 'str',
                'schema_id': 'str',
                'provider': 'str',
                'user_id': 'str',               # inherited from auth (required)
                'domain_id': 'str',             # inherited from auth (required)
            }

        Returns:
            results (list)
            total_count (int)
        """

        query = params.get("query", {})
        user_secret_vos, total_count = self.user_secret_mgr.list_user_secrets(query)

        return user_secret_vos, total_count

    @transaction(
        permission="secret:UserSecret.read",
        role_types=["USER"],
    )
    @check_required(["query", "user_id", "domain_id"])
    @append_query_filter(["user_id", "domain_id"])
    @append_keyword_filter(["user_secret_id", "name", "schema_id", "provider"])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)', # required
                'user_id': 'str',           # inherited from auth (required)
                'domain_id': 'str',         # inherited from auth (required)
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get("query", {})
        return self.user_secret_mgr.stat_user_secrets(query)



import logging
from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel

_LOGGER = logging.getLogger(__name__)


class TrustedSecret(MongoModel):
    trusted_secret_id = StringField(
        max_length=40, generate_id="trusted-secret", unique=True
    )
    name = StringField(max_length=255)
    schema_id = StringField(max_length=40, null=True, default=None)
    provider = StringField(max_length=40, null=True, default=None)
    tags = DictField()
    encrypted = BooleanField(default=False)
    encrypt_options = DictField()
    trusted_account_id = StringField(max_length=40, null=True, default=None)
    resource_group = StringField(max_length=40, choices=("DOMAIN", "WORKSPACE"))
    workspace_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        "updatable_fields": [
            "name",
            "schema_id",
            "tags",
            "encrypted",
            "encrypt_options",
        ],
        "minimal_fields": ["trusted_secret_id", "name", "schema_id", "provider"],
        "ordering": ["name"],
        "indexes": [
            "name",
            "schema_id",
            "provider",
            "trusted_account_id",
            "resource_group",
            "workspace_id",
            "domain_id",
        ],
    }



from spaceone.api.secret.v1 import user_secret_pb2, user_secret_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class UserSecret(BaseAPI, user_secret_pb2_grpc.UserSecretServicer):
    pb2 = user_secret_pb2
    pb2_grpc = user_secret_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "UserSecretService", metadata
        ) as user_secret_service:
            return self.locator.get_info(
                "UserSecretInfo", user_secret_service.create(params)
            )

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "UserSecretService", metadata
        ) as user_secret_service:
            return self.locator.get_info(
                "UserSecretInfo", user_secret_service.update(params)
            )

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "UserSecretService", metadata
        ) as user_secret_service:
            user_secret_service.delete(params)
            return self.locator.get_info("EmptyInfo")

    def update_data(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "UserSecretService", metadata
        ) as user_secret_service:
            user_secret_service.update_data(params)
            return self.locator.get_info("EmptyInfo")

    def get_data(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "UserSecretService", metadata
        ) as user_secret_service:
            user_secret_data = user_secret_service.get_data(params)
            return self.locator.get_info("UserSecretDataInfo", user_secret_data)

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "UserSecretService", metadata
        ) as user_secret_service:
            return self.locator.get_info(
                "UserSecretInfo", user_secret_service.get(params)
            )

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "UserSecretService", metadata
        ) as user_secret_service:
            user_secret_vos, total_count = user_secret_service.list(params)
            return self.locator.get_info(
                "UserSecretsInfo",
                user_secret_vos,
                total_count,
                minimal=self.get_minimal(params),
            )

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "UserSecretService", metadata
        ) as user_secret_service:
            return self.locator.get_info(
                "StatisticsInfo", user_secret_service.stat(params)
            )



import logging
import json
import etcd3
from spaceone.core.connector import BaseConnector

__all__ = ['EtcdConnector']
_LOGGER = logging.getLogger(__name__)


class EtcdConnector(BaseConnector):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.client = etcd3.client(host=self.config.get('host'), port=self.config.get('port'))

    @staticmethod
    def _response_value(response):
        try:
            if response:
                data_bytes = response[0]

                if data_bytes:
                    return json.loads(data_bytes)

            return {}
        except Exception as e:
            _LOGGER.error(f'Response Error: {e}')
            return {}

    def create_secret(self, secret_id, data):
        self.client.put(secret_id, json.dumps(data))

    def delete_secret(self, secret_id):
        self.client.delete(secret_id)

    def update_secret(self, secret_id, data):
        self.delete_secret(secret_id)
        self.create_secret(secret_id, data)

    def get_secret(self, secret_id):
        return self._response_value(self.client.get(secret_id))



import logging

from spaceone.core.manager import BaseManager
from spaceone.secret.model.trusted_secret_model import TrustedSecret

_LOGGER = logging.getLogger(__name__)


class TrustedSecretManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.trusted_secret_model: TrustedSecret = self.locator.get_model(
            "TrustedSecret"
        )

    def create_trusted_secret(self, params):
        def _rollback(trusted_secret_vo):
            _LOGGER.info(
                f"[ROLLBACK] Delete trusted secret : {trusted_secret_vo.name} ({trusted_secret_vo.trusted_secret_id})"
            )
            trusted_secret_vo.delete()

        trusted_secret_vo: TrustedSecret = self.trusted_secret_model.create(params)

        self.transaction.add_rollback(_rollback, trusted_secret_vo)

        return trusted_secret_vo

    def update_trusted_secret_by_vo(self, params, trusted_secret_vo):
        def _rollback(old_data):
            _LOGGER.info(
                f'[ROLLBACK] Revert Data : {old_data["name"]} ({old_data["trusted_secret_id"]})'
            )
            trusted_secret_vo.update(old_data)

        self.transaction.add_rollback(_rollback, trusted_secret_vo.to_dict())

        return trusted_secret_vo.update(params)

    @staticmethod
    def delete_trusted_secret_by_vo(trusted_secret_vo):
        trusted_secret_vo.delete()

    def get_trusted_secret(self, trusted_secret_id, domain_id, workspace_id=None):
        conditions = {
            "trusted_secret_id": trusted_secret_id,
            "domain_id": domain_id,
        }

        if workspace_id:
            conditions["workspace_id"] = workspace_id

        return self.trusted_secret_model.get(**conditions)

    def filter_trusted_secrets(self, **conditions):
        return self.trusted_secret_model.filter(**conditions)

    def list_trusted_secrets(self, query):
        return self.trusted_secret_model.query(**query)

    def stat_trusted_secrets(self, query):
        return self.trusted_secret_model.stat(**query)



import logging
from pymongo import MongoClient
from spaceone.core.connector import BaseConnector


__all__ = ['MongoDBConnector']
_LOGGER = logging.getLogger(__name__)


class MongoDBConnector(BaseConnector):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        client = MongoClient(**self.config)
        db = client.secret_data
        self.secret_data = db.secret_data

    def create_secret(self, secret_id, data):
        _document = {'secret_id': secret_id, 'data': data}
        return self.secret_data.insert_one(_document)

    def delete_secret(self, secret_id):
        self.secret_data.delete_one({'secret_id': secret_id})

    def update_secret(self, secret_id, data):
        _query = {'secret_id': secret_id}
        self.secret_data.update_one(_query, {'$set': {'data': data}})

    def get_secret(self, secret_id):
        secret_data_info = self.secret_data.find_one({'secret_id': secret_id})
        return secret_data_info.get('data', {})





from spaceone.secret.connector.aws_secret_manager_connector import *
from spaceone.secret.connector.vault_connector import *
from spaceone.secret.connector.consul_connector import *
from spaceone.secret.connector.identity_connector import *
from spaceone.secret.connector.mongodb_connector import *



from spaceone.api.secret.v1 import trusted_secret_pb2, trusted_secret_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class TrustedSecret(BaseAPI, trusted_secret_pb2_grpc.TrustedSecretServicer):

    pb2 = trusted_secret_pb2
    pb2_grpc = trusted_secret_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('TrustedSecretService', metadata) as trusted_secret_service:
            return self.locator.get_info('TrustedSecretInfo', trusted_secret_service.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('TrustedSecretService', metadata) as trusted_secret_service:
            return self.locator.get_info('TrustedSecretInfo', trusted_secret_service.update(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('TrustedSecretService', metadata) as trusted_secret_service:
            trusted_secret_service.delete(params)
            return self.locator.get_info('EmptyInfo')

    def update_data(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('TrustedSecretService', metadata) as trusted_secret_service:
            trusted_secret_service.update_data(params)
            return self.locator.get_info('EmptyInfo')

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('TrustedSecretService', metadata) as trusted_secret_service:
            return self.locator.get_info('TrustedSecretInfo', trusted_secret_service.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('TrustedSecretService', metadata) as trusted_secret_service:
            trusted_secret_vos, total_count = trusted_secret_service.list(params)
            return self.locator.get_info('TrustedSecretsInfo', trusted_secret_vos, total_count,
                                         minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('TrustedSecretService', metadata) as trusted_secret_service:
            return self.locator.get_info('StatisticsInfo', trusted_secret_service.stat(params))
